#ifndef RL_Animation_H
#define RL_Animation_H 1

/***************************************************************************/

#include "main.h"
#include "rl_time.h"

/***************************************************************************/

/*
** All available animation modes 
**
** NOTE: If you insert, remove or swap entries here, you need
** to adapt the matching TAB_CYLE entry in rl_wifi.cpp, or the
** web interface wont match the modes anymore.
*/

enum{
  ANIMATION_STATIC = 0,           /* show a fixed color */
  ANIMATION_RAINBOW,              /* rotating Rainbow */
  ANIMATION_RAINBOWSPARKLE,       /* rotating Rainbow with sparkles */
  ANIMATION_FIREPLACE,            /* fire place style */
  ANIMATION_BLUEFIRE,             /* ice style fire */
  ANIMATION_GREENFIRE,            /* grass style fire */
  ANIMATION_RGBFADE,              /* fade between Red, Green, Blue */
  ANIMATION_TWINKLE,              /* change random dot with random RGB */
  ANIMATION_RACE,                 /* Trailing pixels racing "randomly" */
  ANIMATION_RACELOOP,             /* Trailing pixels looping "randomly" */
  ANIMATION_COLORCOMET,           /* flying comet */
  ANIMATION_COLORCOMETSTATIC,     /* flying comet single color */
  ANIMATION_RANDOM,               /* cycle randomly though our favorites */
#if HARDWARE_NUMBEROF_LEDSTRIPS > 1 
  ANIMATION_RANDOMSYNC,           /* cycle randomly though our favorites, all animations save effect */
#endif
  ANIMATION_LAST                  /* no real mode, keep at the end */
};

#if HARDWARE_NUMBEROF_LEDSTRIPS <= 1
#define ANIMATION_RANDOMSYNC ANIMATION_RANDOM
#endif

#define ANIMATION_RANDOM_MODEFLAGS     ( 1 << ANIMATION_RAINBOW ) | ( 1 << ANIMATION_FIREPLACE ) | ( 1 << ANIMATION_RGBFADE ) | ( 1 << ANIMATION_RACE ) | ( 1 << ANIMATION_RACELOOP ) | ( 1 << ANIMATION_COLORCOMET )

/***************************************************************************/

#define ANIMATION_FADESTEP          16  /* x/256 color steps */
#define ANIMATION_FADEDELAY         50  /* delay in FPS */

#define ANIMATION_BRIGHTNESSUP     180  /* for animation effects using a brigher base color */
#define ANIMATION_BRIGHTNESSDOWN    20  /* for animation effects using a darker base color */

/* twinkle configuration */

#define TWINKLE_FADEAWAY 4

/* fire place, ice fire place configuration */

#define FIRE_SPEED                   2    /* animation speed */

#define FIRE_COOLING                55    /* cell cooling */
#define FIRE_SPARKING               35    /* sparks thresshold */
#define FIRE_SPARKS                  4    /* number of sparks */
#define FIRE_SPARKHEIGHT             3    /* radius of the ignite zone */

#define FIRE_BLENDSELF               2
#define FIRE_BLENDNEIGHBOR1          3
#define FIRE_BLENDNEIGHBOR2          2
#define FIRE_BLENDNEIGHBOR3          1
#define FIRE_BLENDDIVIDER           ( FIRE_BLENDSELF + FIRE_BLENDNEIGHBOR1 + FIRE_BLENDNEIGHBOR2 + FIRE_BLENDNEIGHBOR3 )

/* rainbow sparkle */

#define RAINBOW_SPARKLERATE         25   /* percentage change of having a sparkle */

/* race configuration */

#define RACE_NUMBEROF                4
#define RACE_TRAIL_FADEAWAY         ( 100 - ( ( ( HARDWARE_SEGMENT_1_LEDS > 30 ) && ( HARDWARE_SEGMENT_1_LEDS < 70 ) ) ? HARDWARE_SEGMENT_1_LEDS : 40 ) )

/* color comet (static) configuration */

#define COLORCOMET_BPM              14
#define COLORCOMET_TRAIL_FADEAWAY   27

/***************************************************************************/

/*
** fader control data 
*/

struct RL_AnimationFader {
	uint16_t  Flags;                           /* is this animation active? */
	 int16_t  FadeBrightness;                  /* fading when changing animation mode (0-100%) */
	uint16_t  FadeDelay;                       /* blank time between fading */
};

/*
** some flags for animation fading control
*/

#define AFF_ACTIVE              ( 1 << 0 ) /* running */
#define AFF_FADEIN              ( 1 << 1 ) /* fade in state */
#define AFF_FADEOUT             ( 1 << 2 ) /* fade out state */
#define AFF_FADEDELAY           ( 1 << 3 ) /* black pause state */

#define AFF_MODEMASK            ( AFF_FADEIN | AFF_FADEOUT | AFF_FADEDELAY )

/*
** RL_Animation
**
** keeps constant and running animation data
*/


struct RL_Animation {
/* general data */
	uint32_t *AnimationMode;                     /* pointer to config for this strip */
	uint32_t  NumberOfLEDs;                      /* number of LEDs in this strip */
	CRGB     *RenderBuffer;                      /* buffer for this LED strip */
/* general animation data */
//	uint16_t  Flags;                             /* */
	uint16_t  BufferedMode;                      /* buffered rlc.rlc_AnimationMode */
	uint16_t  BufferedRandomMode;                /* buffered rlc.rlc_AnimationMode (random/randomsync) */
	uint16_t  Phase;                             /* current step of the animation, usage depends on type of animation */
	uint16_t  InternalCount1;                    /* depends on type of animation */
/* animation mode specific */
	uint16_t  RaceSpeedInit[ RACE_NUMBEROF ]  ;  /* race loop speeds */
	uint16_t  RaceSpeed[ RACE_NUMBEROF ];        /* race loop speed count  */
	uint16_t  RacePos[ RACE_NUMBEROF ];          /* race loop pos */
	byte      Heat[ HARDWARE_LONGEST_LEDSTRIP ]; /* must be the biggest strip or we are doomed */
};

/***************************************************************************/

/*
** prototypes
*/

void     Animation_Setup( void );
void     Animation_Randomizer( void );
uint32_t Animation_GetBrightness( void );
void     Animation_Animate( void );

/***************************************************************************/

#endif /* RL_Animation_H */
