/*
**   rl_fauxmo.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_fauxmo.h"
#include "rl_time.h"
#include "rl_wifi.h"

#include <fauxmoESP.h>

/***************************************************************************/

#if ENABLE_FAUXMO

/***************************************************************************/

fauxmoESP fauxmo;

static char currentdevicename[ FAUXMO_DEVICENAME_SIZEOF + 2 ];   /* buffer to detect name changes */

/***************************************************************************/

/* /// Fauxmo_Setup()
**
*/

/***************************************************************************/

void Fauxmo_Setup( void )
{
	debug(( "\nFauxmo_Startup()\n" ));

/* no device name -> user disabled alexa support */

	if( rlc.rlc_FauxmoDeviceName[0] ) {

		fauxmo.createServer( true );

		fauxmo.addDevice( rlc.rlc_FauxmoDeviceName );

		strcpy( currentdevicename, rlc.rlc_FauxmoDeviceName );   /* store to detect name changes */

		fauxmo.setPort(80);   /* port 80 is required for devices Gen2 and up */

		fauxmo.enable( true );

		fauxmo.onSetState([](unsigned char device_id, const char * device_name, bool state, unsigned char value ) {
			debug(( "\nFauxmo__Handle() - Device #%d (%s) state: %s value: %d\n", device_id, device_name, ( state ? "ON" : "OFF" ), value ));

			if( !( strcmp( device_name, FAUXMO_DEVICENAME ) ) ) {
				debug(( "Fauxmo__Handle() - %s - State: %s - Brightness: %d%% (%d)\n", device_name, ( state ? "ON" : "OFF" ), FAUXMO_VALUECONVERT, value ));
				if( state ) {
					rlrd.rlrd_GlobalMode = RLRD_GLOBALMODE_ALWAYSON;
				} else {
					rlrd.rlrd_GlobalMode = RLRD_GLOBALMODE_ALWAYSOFF;
				}
				debug(( "Fauxmo__Handle() - Switching %s: %s\n", device_name, ( ( rlrd.rlrd_GlobalMode == RLRD_GLOBALMODE_ALWAYSON ) ? "ON" : "OFF" ) ));

				/* telling alexa to switch to 99% brightness should turn global auto mode back on.
				** This is a little hack as there is IMHO no other may to add more states without
				** creating more complex alexa rules or defining multiple devices for the same 
				** thing.
				*/
				if( ( value != FAUXMO_AUTOSWITCH_BRIGHTNESS ) ) {
					debug(( "Fauxmo__Handle() - Settings brightness to %d%% (%d)\n", FAUXMO_VALUECONVERT, value ));
					rlc.rlc_Brightness   = FAUXMO_VALUECONVERT; /* convert alexa brightness value (0-254) to percent (0-100) */
				} else {
					debug(( "Fauxmo__Handle() - Turning auto mode back on: %d%% (%d)\n", FAUXMO_VALUECONVERT, value ));
					rlrd.rlrd_GlobalMode = RLRD_GLOBALMODE_AUTO;  /* turn our ring back to auto mode */
				}
			}
		});
	}
}
/* \\\ Fauxmo_Setup() */

/* /// Fauxmo_Handle()
**
*/

/***************************************************************************/

void Fauxmo_Handle( void )
{

	debugex(( "\nFauxmo_Handle()\n" ));

	if( rlc.rlc_FauxmoDeviceName[0] ) {

		fauxmo.handle();

	}

}
/* \\\ Fauxmo_Handle */
/* /// Fauxmo_Rename()
**
*/

/***************************************************************************/

void Fauxmo_Rename( void )
{

	debugex(( "\nFauxmo_Rename()\n" ));

	if( rlc.rlc_FauxmoDeviceName[0] ) {

		/* device name strings are different */
		debug(( "Fauxmo_Rename() - Renaming '%s' to '%s'\n", currentdevicename, rlc.rlc_FauxmoDeviceName ));

		fauxmo.renameDevice( currentdevicename, rlc.rlc_FauxmoDeviceName );

		strcpy( currentdevicename, rlc.rlc_FauxmoDeviceName );   /* store old name to be able to rename */
	}

}
/* \\\ Fauxmo_Rename */
/* /// Fauxmo_Disable()
**
*/

/***************************************************************************/

void Fauxmo_Disable( void )
{

	debugex(( "\nFauxmo_Disable()\n" ));

	fauxmo.enable( true );

}
/* \\\ Fauxmo_Rename */

/***************************************************************************/

#endif /* ENABLE_FAUXMO */