/*
**   rl_global.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_animation.h"
#include "rl_display.h"
#include "rl_global.h"
#include "rl_sound.h"
#include "rl_testmode.h"
#include "rl_time.h"
#include "rl_wifi.h"

/***************************************************************************/

/* /// Global_Setup()
**
*/

/***************************************************************************/

void Global_Setup( void )
{
	debug(( "\nGlobal_Startup()\n" ));

	Global_SettingsReset();

	if( !SPIFFS.begin( true ) ) {
		debug(( "SPIFFS Mount Failed\n" ));
	}

}
/* \\\ */

/* /// Global_UpdateDisplayBrightness()
**
** dim device at night time
**
*/

/***************************************************************************/

void Global_UpdateDisplayBrightness( void  ) 
{
uint32_t brightness = rlc.rlc_DisplayBrightnessDay;
#if ENABLE_DEBUG
uint32_t day = 1;   /* flag to only show debug, when needed and with day/night information */
#endif
	/* if nightstart and nightend hours are the same, brightness is locked to the day time settings */
	if( rlc.rlc_DisplayBrightnessNightStart != rlc.rlc_DisplayBrightnessNightEnd ) {
		if( rlc.rlc_DisplayBrightnessNightStart > rlc.rlc_DisplayBrightnessNightEnd ) {
			if( ( timecurrent.tm_hour >= rlc.rlc_DisplayBrightnessNightStart ) || ( timecurrent.tm_hour < rlc.rlc_DisplayBrightnessNightEnd ) ) {
				brightness = rlc.rlc_DisplayBrightnessNight;
#if ENABLE_DEBUG
				day = 0;
#endif
			}
		} else {
			if( ( timecurrent.tm_hour >= rlc.rlc_DisplayBrightnessNightStart ) && ( timecurrent.tm_hour < rlc.rlc_DisplayBrightnessNightEnd ) ) {
				brightness = rlc.rlc_DisplayBrightnessNight;
#if ENABLE_DEBUG
				day = 0;
#endif
			}
		}
	}

/* for stats screen */
	rlrd.rlrd_Flags &= ~RLRDF_STATUS_DISPLAYBRIGHTNESSNIGHT;
	if( ( brightness == rlc.rlc_DisplayBrightnessNight ) ) {
		rlrd.rlrd_Flags |= RLRDF_STATUS_DISPLAYBRIGHTNESSNIGHT;
	}

/* incorporate the general brightness */
	brightness = brightness * rlc.rlc_Brightness / 100;
	brightness = brightness * Animation_GetBrightness()  / 100;

	/* only update brightness when changed */
	if( rlrd.rlrd_GlobalDisplayBrightness != brightness ) {
		rlrd.rlrd_GlobalDisplayBrightness = brightness;
		FastLED.setBrightness( 255 * brightness / 100 );
		debug(( "\nGlobal_ControlDisplayBrightness() - Switching to %s Time\n", ( day ? "Day" : "Night" ) ));
	}
}
/* \\\ */

/* /// Global_UpdatePowerState()
**
** turn off the device during night time
**
*/

/***************************************************************************/

void Global_UpdatePowerState( void )
{

	rlrd.rlrd_Flags &= ~RLRDF_STATUS_POWERSTATENIGHT;

	/* if nightstart and nightend hours are the same, power state is locked to day time settings */
	if( rlc.rlc_PowerStateNightStart != rlc.rlc_PowerStateNightEnd ) {
		if( rlc.rlc_PowerStateNightStart > rlc.rlc_PowerStateNightEnd ) {
			if( ( timecurrent.tm_hour >= rlc.rlc_PowerStateNightStart ) || ( timecurrent.tm_hour < rlc.rlc_PowerStateNightEnd ) ) {
				rlrd.rlrd_Flags      |= RLRDF_STATUS_POWERSTATENIGHT;
				debug(( "Global_ControlPowerState() - Night Time\n" ));
#if ENABLE_DEBUG
			} else {
				debug(( "Global_ControlPowerState() - Day Time\n" ));
#endif
			}
		} else {
			if( ( timecurrent.tm_hour >= rlc.rlc_PowerStateNightStart ) && ( timecurrent.tm_hour < rlc.rlc_PowerStateNightEnd ) ) {
				rlrd.rlrd_Flags |= RLRDF_STATUS_POWERSTATENIGHT;
				debug(( "Global_ControlPowerState() - Night Time\n" ));
#if ENABLE_DEBUG
			} else {
				debug(( "Global_ControlPowerState() - Day Time\n" ));
#endif
			}
		}
	}
}
/* \\\ Global_UpdatePowerState() */
/* /// Global_UpdateVolume()
**
** update sound volume depending on the time of day
**
*/

/***************************************************************************/

void Global_UpdateVolume( void )
{
uint32_t soundvolume = rlc.rlc_SoundVolumeDay;
	/* if nightstart and nightend hours are the same, volume is set to day time settings */
	if( rlc.rlc_SoundVolumeNightStart != rlc.rlc_SoundVolumeNightEnd ) {
		if( rlc.rlc_SoundVolumeNightStart > rlc.rlc_SoundVolumeNightEnd ) {
			if( ( timecurrent.tm_hour >= rlc.rlc_SoundVolumeNightStart ) || ( timecurrent.tm_hour < rlc.rlc_SoundVolumeNightEnd ) ) {
				soundvolume = rlc.rlc_SoundVolumeNight;
				debug(( "Global_UpdateVolume() - Night Time\n" ));
#if ENABLE_DEBUG
			} else {
				debug(( "Global_UpdateVolume() - Day Time\n" ));
#endif
			}
		} else {
			if( ( timecurrent.tm_hour >= rlc.rlc_SoundVolumeNightStart ) && ( timecurrent.tm_hour < rlc.rlc_SoundVolumeNightEnd ) ) {
				soundvolume = rlc.rlc_SoundVolumeNight;
				debug(( "Global_UpdateVolume() - Night Time\n" ));
#if ENABLE_DEBUG
			} else {
				debug(( "Global_UpdateVolume() - Day Time\n" ));
#endif
			}
		}
	} else {
		soundvolume = rlc.rlc_SoundVolumeDay;
		debug(( "Global_UpdateVolume() - Day Time\n" ));
	}
/* for stats screen */
	rlrd.rlrd_Flags &= ~RLRDF_STATUS_SOUNDVOLUMENIGHT;
	if( ( soundvolume == rlc.rlc_SoundVolumeNight ) ) {
		rlrd.rlrd_Flags |= RLRDF_STATUS_SOUNDVOLUMENIGHT;
	}

	/* Set the volume. If the volume is the same nothing will be done */
	Sound_SetVolume( soundvolume );
}
/* \\\ Global_UpdateVolume */
/* /// Global_HandleEvents() 
**
** force is a sneaky way to update everything on settings change
** without calling everything each time seperately
**
*/

/***************************************************************************/

void Global_HandleEvents( bool force )
{

	if( ( rlc.rlc_LEDColorCorrection != rlrd.rlrd_LEDColorCorrection ) || ( rlc.rlc_LEDColorTemperature != rlrd.rlrd_LEDColorTemperature ) || force ) {
		rlrd.rlrd_LEDColorCorrection  = rlc.rlc_LEDColorCorrection;
		rlrd.rlrd_LEDColorTemperature = rlc.rlc_LEDColorTemperature;
		Display_UpdateSettings();
	}

/* update display brightness depending on night time settings */

	Global_UpdateDisplayBrightness();  /* alexa command may change brightness */

/*** events happening every hour are done here ***/

	if( ( timecurrent.tm_hour != timereference.tm_hour ) || force ) {

/* update power state depending on night time settings */

		Global_UpdatePowerState();

/* update volume depending on night time settings */

		Global_UpdateVolume();

	/* play sound after updating volume, as we may be in night mode now */

		if( !force ) { /* no sound in force mode */
			if( ( rlrd.rlrd_Flags & RLRDF_STATUS_STARTSOUNDDONE ) ) {
				Sound_Play( rlc.rlc_SoundIDHour, false );
			} else {
				rlrd.rlrd_Flags |= RLRDF_STATUS_STARTSOUNDDONE;    /* playing the start sound is a once in a lifetime event */
				Sound_Play( rlc.rlc_SoundIDBoot, false );
			}
		}

	}

/*** events happening every quarter hour are done here ***/

	if( ( timecurrent.tm_min != timereference.tm_min ) || force ) { /* minute changed */

		switch( timecurrent.tm_min ) {
			case 30:
				if( !force ) { /* no sound in force mode */
					Sound_Play( rlc.rlc_SoundIDHalfHour, false );
				}
				//break; /* fall through */
			case 0:
			case 15:
			case 45:
				Animation_Randomizer();
				break;
		}
	}
}
/* \\\ Global_HandleEvents */

/* /// Global_SettingsSave()
**
*/

/***************************************************************************/

uint32_t Global_SettingsSave( void )
{
uint32_t result = sizeof( struct RingLightConfig );

	debug(( "\nGlobal_SettingsSave()...\n" ));

	File datafile = SPIFFS.open( SETTINGS_NAME, "w" );

	rlc.rlc_Version    = RLC_VERSION;    /* update to version information */
	rlc.rlc_Size       = RLC_SIZE;       /* update to current size information */

	if( datafile ) {

		debug(( "Saving settings...\n" ));

		result -= datafile.write( (const uint8_t *) &rlc, result );

		datafile.close();

		debug(( "Settings saved... (%s)\n", result ? "FAILED" : "OK" ));
	}
	return( result );
}
/* \\\ */
/* /// Global_SettingsLoad()
**
*/

/***************************************************************************/

uint32_t Global_SettingsLoad( void )
{
uint32_t result = sizeof( struct RingLightConfig );

	debug(( "\nGlobal_SettingsLoad()...\n" ));

	File datafile = SPIFFS.open( SETTINGS_NAME, "r" );

	if( datafile ) {

		debug(( "Global_SettingsLoad() - Loading\n" ));

		result -= datafile.read( (uint8_t *) &rlc, result );

		datafile.close();

		if( result || ( rlc.rlc_Version != RLC_VERSION ) || ( rlc.rlc_Size != RLC_SIZE )  ) {
			Global_SettingsReset();   /* load error or wrong version, so reset config */
			result = 1;
		}

		debug(( "Settings loaded... (%s)\n", result ? "FAILED" : "OK" ));
	}
	return( result );
}
/* \\\ */
/* /// Global_SettingsReset()
**
** reset global device configuration to factory defaults
**
*/

/***************************************************************************/

void Global_SettingsReset()
{
struct RingLightConfig *config = &rlc;

	debug(( "\nGlobal_SettingsReset() - Resetting settings...\n" ));

	/* clear all data first just in case we forgot something important */

	memset( config, 0, sizeof( struct RingLightConfig ) );

	config->rlc_Version                     = RLC_VERSION;
	config->rlc_Size                        = RLC_SIZE;
	config->rlc_Flags                       = RLCF_SOUNDENABLE;
	config->rlc_AnimationMode1              = ANIMATION_RANDOM;
#if HARDWARE_NUMBEROF_LEDSTRIPS > 1 
	config->rlc_AnimationMode2              = ANIMATION_RANDOMSYNC;
#endif
	config->rlc_AnimationModeRandom         = ANIMATION_RANDOM_MODEFLAGS;
	config->rlc_Brightness                  = 50;
	config->rlc_RGBColor                    = CRGB::Blue;
	config->rlc_RGBColorAlternate           = CRGB(109,100,215);

	config->rlc_DisplayBrightnessNightStart = DISPLAY_BRIGHTNESS_NIGHTSTART;
	config->rlc_DisplayBrightnessNightEnd   = DISPLAY_BRIGHTNESS_NIGHTEND;
	config->rlc_DisplayBrightnessDay        = DISPLAY_BRIGHTNESS_DAY; 
	config->rlc_DisplayBrightnessNight      = DISPLAY_BRIGHTNESS_NIGHT;
	config->rlc_DisplayStatusLEDMode        = DISPLAY_STATUSLEDMODE_OFF;

	config->rlc_SoundVolumeNightStart       = SOUND_VOLUME_NIGHTSTART;
	config->rlc_SoundVolumeNightEnd         = SOUND_VOLUME_NIGHTEND;
	config->rlc_SoundVolumeDay              = SOUND_VOLUME_DAY; 
	config->rlc_SoundVolumeNight            = SOUND_VOLUME_NIGHT;

	config->rlc_LEDColorCorrection          = 2;                  /* Typical LED Strip */
	config->rlc_LEDColorTemperature         = 2;                  /* Tungsten40W */

	config->rlc_PowerStateNightStart        = POWER_STATE_NIGHTSTART;
	config->rlc_PowerStateNightEnd          = POWER_STATE_NIGHTEND;

	config->rlc_SoundIDBoot                 = SOUNDID_STARTUP_2;
	config->rlc_SoundIDHour                 = SOUNDID_BELL_1;
	config->rlc_SoundIDHalfHour             = SOUNDID_BELL_2;

	strcpy( config->rlc_WifiSSID         , WIFI_SSID          );  /* we use the config from credentials.h file */
	strcpy( config->rlc_WifiPass         , WIFI_PASSWORD      );  /* we use the config from credentials.h file */

	strcpy( config->rlc_WebServerPassword, WEBSERVER_PASSWORD );  /* we use the config from credentials.h file */

	strcpy( config->rlc_FauxmoDeviceName , FAUXMO_DEVICENAME  );  /* we use the config from credentials.h file */
	strcpy( config->rlc_HostName         , WIFI_HOSTNAME      );  /* we use the config from credentials.h file */

}
/* \\\ */