/*
**   rl_html.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include <WebServer.h>

#include "main.h"
#include "rl_html.h"
#include "rl_webserver.h"

/***************************************************************************/

/*
** YES/NO selector 
*/

static const char *TAB_YESNO_YESNO[] PROGMEM = {
	"Yes",
	"No",
	NULL
};

/***************************************************************************/

/*
** ON/OFF selector
*/

static const char *TAB_ONOFF_ONOFF[] PROGMEM = {
	"On",
	"Off",
	NULL
};

/***************************************************************************/

/* /// HTML_CreateTD()
**
*/

/***************************************************************************/

static void HTML_CreateTD( uint32_t flags )
{
	htmlpage += "<td style=\"text-align:";

	if( flags & WSF_C ) {
		htmlpage += "center\">";
	} else {
		if( flags & WSF_R ) {
			htmlpage += "right\">";
		} else {
			htmlpage += "left\">";
		}
	}
}
/* \\\ */

/* /// HTML_CharToHex()
**
*/

/***************************************************************************/

static byte HTML_CharToHex( char chr ) 
{
byte val = 0;

	if( ( chr >= '0' ) && ( chr <= '9' ) ) {
		val = chr - 48;
	} else {
		if( ( chr >='a' ) && ( chr <= 'f' ) ) {
			val = chr - 97 + 10;
		} else {
			if( ( chr >= 'A' ) && ( chr <= 'F' ) ) {
				val = chr - 65 + 10;
			}
		}
	}
	return( val );
}
/* \\\ HTML_CharToHex */
/* /// HTML_CreateUIFromTable()
**
*/


/***************************************************************************/

bool HTML_CreateUIFromTable( uint32_t pageid, struct ws_Settings *tab_uidata )
{
uint32_t i, idx, value, tr = 0, td = 0;
bool        update = false;
char       *cyclestr;
#define BUFFER_SIZEOF 512
char        buf[BUFFER_SIZEOF];
#define OACTION_SIZEOF 64
char        oaction[OACTION_SIZEOF];
String      serverarg;         /* cache for serverarg */
const char *oid;               /* cache for object id */
const char *oname;             /* cache for object name */
const char *odisabled;         /* cache for disabled */
const char *ohidden;           /* cache for hidden */

	debug(( "\nWebServer_CreateUIFromTable() - %d\n", pageid ));

	htmlpage += "<table border=\"0\">";

	for( i = 0 ; WS_TYPE != WSTYPE_INVALID ; i++ ) {
		/* object visible in this contect? */
		if( !(WS_PAGEID & pageid ) ) {
			continue; /* save us time and check next in list */
		}
		oid        = ( WS_ID   ? WS_ID   : "" );             /* create valid object id */
		oname      = ( WS_NAME ? WS_NAME : "" );             /* create valid object name */
		serverarg  = server.arg( oid );                      /* create valid object server arg */

		odisabled = "";
		if( WS_FLAGS & WSF_D ) {
			odisabled = " disabled";
		}
		ohidden = "";
		if( WS_FLAGS & WSF_D ) {
			ohidden = " hidden";
		}

		if( !WS_ACTION ) {                                  /* create valid object action */
			const char *actiontype = "onclick";
			const char *actionfunc = "NM";
			#define DOM_ACTION "%s=\"%s('%s')\""
			switch( WS_TYPE ) {
				case WSTYPE_BUTTON:
				case WSTYPE_CHECKBOX:
				case WSTYPE_REFFLAG:
					actionfunc = "CB";
					break;
				default: 
					actiontype = "onchange";
					break;
			}
			snprintf( oaction, OACTION_SIZEOF-1, DOM_ACTION, actiontype, actionfunc, oid );
		} else {
			snprintf( oaction, OACTION_SIZEOF-1, WS_ACTION, oid );
		}

		debugex(( "HTML_CreateUIFromTable() - idx: %d, Type: %d\n", i, WS_TYPE));
		if( !tr ) {
			htmlpage += "<tr>"; /* new row */
			tr++;
		}
		switch( WS_TYPE ) {
/* /// WSTYPE_TITLE
*/
/***************************************************************************/
			case WSTYPE_TITLE:
				htmlpage += "<td colspan=\"2\" style=\"color:black; text-align:center\" ><h2>";
				htmlpage += oname;
				htmlpage += "</h2></td>";
				break;
/* \\\ WSTYPE_TITLE  */
/* /// WSTYPE_SPACER
*/
/***************************************************************************/
			case WSTYPE_SPACER:
				htmlpage += "<td colspan=\"2\" style=\"color:black; text-align:center\" ></td>";
				break;
/* \\\ WSTYPE_SPACER */
/* /// WSTYPE_BUTTON
*/
/***************************************************************************/
			case WSTYPE_BUTTON:
				debugex(( "HTML_CreateUIFromTable() - WSTYPE_BUTTON\n" ));
				if( ( serverarg != nullptr ) ) {
					webserver_action = WS_BUTTONCOMMAND;
					debugex(( "HTML_CreateUIFromTable() - Setting action %d\n", webserver_action ));
				} /* serverarg */
				if( !td ) {
					HTML_CreateTD( WS_FLAGS );
					td++;
				}

#define DOM_BT "<button id=\"%s\" name=\"%s\" value=\"1\" %s %s>%s</button>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_BT, oid, oid, odisabled, oaction, oname );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_BUTTON */
/* /// WSTYPE_CHECKBOX
*/
/***************************************************************************/
			case WSTYPE_CHECKBOX:
				if( ( serverarg != nullptr ) ) {
					debug(( "HTML_CreateUIFromTable() - WSTYPE_CHECKBOX: Arg is %s\n", serverarg.c_str() ));
					if( ( serverarg == "1"  ) ) {
						*WS_FLAGPTR |=  WS_FLAGMASK;
					} else {
						*WS_FLAGPTR &= ~WS_FLAGMASK;
					}
				}
				htmlpage += "<td></td>";
				HTML_CreateTD( WSF_L );

#define DOM_CB "<input id=\"%s\" %s type=\"checkbox\" %s><label for=\"%s\">%s</label><br>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_CB, oid, (*WS_FLAGPTR & WS_FLAGMASK ) ? "checked" : "", oaction, oid, oname );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_CHECKBOX */ 
/* /// WSTYPE_CYCLE
*/
/***************************************************************************/
			case WSTYPE_CYCLE:
				if( ( serverarg != nullptr ) ) {
					debug(( "HTML_CreateUIFromTable() - WSTYPE_CYCLE: Arg is %s\n", serverarg.c_str() ));
					for( idx = 0 ; ( cyclestr = WS_CYCLEARRAY[ idx ] ) ; idx++ ) {
						if( serverarg == cyclestr ) {
							*WS_CYCLEACTIVE =  idx; /* store index */
						}
					}
					update = true;
				}
				/* build web page here */
				HTML_CreateTD( WSF_R );
				htmlpage += oname;
				htmlpage += "</td>";
				HTML_CreateTD( WSF_L );

#define DOM_CY1 "<select id=\"%s\" %s>"
#define DOM_CY2 "<option value=\"%s\"%s>%s</option>"
#define DOM_CY3 "</select>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_CY1, oid, oaction );
				htmlpage += buf;
				/* add all options to our cycle gadget */
				for( idx = 0 ; ( cyclestr = WS_CYCLEARRAY[ idx ] ) ; idx++ ) {
					snprintf( buf, BUFFER_SIZEOF-1, DOM_CY2, cyclestr, ( ( ( *WS_CYCLEACTIVE ) == idx ) ? " selected" : "" ), cyclestr );
					htmlpage += buf;
				}
				htmlpage += DOM_CY3;
				break;
/* \\\ WSTYPE_CYCLE */
/* /// WSTYPE_SLIDER
*/
/***************************************************************************/
			case WSTYPE_SLIDER:
				debugex(( "HTML_CreateUIFromTable() - WSTYPE_SLIDER\n" ));
				if( ( serverarg != nullptr ) ) {
					update = true;
					value = serverarg.toInt();
					debug(( "HTML_CreateUIFromTable() - WSTYPE_SLIDER: Arg is %ld\n", (long int) value ));
					/* make sure the new value is within the allowed limits */
					if( value > WS_SLIDERMAX ) {
						value = WS_SLIDERMAX;
					}
					if( value < WS_SLIDERMIN ) {
						value = WS_SLIDERMIN;
					}
					*WS_SLIDERPTR = value;
				}
				/* build web page here */
				HTML_CreateTD( WSF_R );
				htmlpage += oname;
				sprintf( buf, " %-3ld", (long int) *WS_SLIDERPTR );
				htmlpage += buf;
				htmlpage += "</td>";
				HTML_CreateTD( WSF_L );

#define DOM_SL  "<input type=\"range\" id=\"%s\" name=\"%s\" value=%ld min=%ld max=%ld step=%ld %s>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_SL, oid, oid, (long int) *WS_SLIDERPTR, (long int) WS_SLIDERMIN, (long int) WS_SLIDERMAX, (long int) WS_SLIDERSTEP, oaction );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_SLIDER */
/* /// WSTYPE_TEXT / PASSWORD
*/
/***************************************************************************/
			case WSTYPE_TEXT:
			case WSTYPE_PASSWORD:
#if ENABLE_DEBUGEXTENDED
				if( WS_TYPE == WSTYPE_TEXT ) {
					debugex(( "HTML_CreateUIFromTable() - WSTYPE_TEXT\n" ));
				} else {
					debugex(( "HTML_CreateUIFromTable() - WSTYPE_PASSWORD\n" ));
				}
#endif
				if( ( ( serverarg != nullptr ) && WS_TEXT != NULL ) ) {
					debugex(( "HTML_CreateUIFromTable() - WSTYPE_TEXT/PASSWORD: Arg is %s\n", serverarg.c_str() ));
					/* make sure the new value is within the allowed limits */
					{ 
					int maxlen = WS_TEXTMAXLEN;
						strncpy( WS_TEXT, serverarg.c_str(), maxlen );
						WS_TEXT[ serverarg.length() ] = 0x00; /* this is safe as we reserved an extra byte */
					}
				}
				/* build web page here */
				if( !( WS_FLAGS & WSF_H ) ) {
					HTML_CreateTD( WSF_R );
					htmlpage += oname;
					htmlpage += "</td>";
					HTML_CreateTD( WSF_L );
				} else {
				}
#define DOM_TX  "<input type=\"%s\" id=\"%s\" name=\"%s\" value=\"%s\" maxlength=\"%d\" %s %s>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_TX, (!( WS_FLAGS & WSF_H ) ? (( WS_TYPE == WSTYPE_TEXT ) ? "text" : "password" ) : "hidden"), oid, oid, (WS_TEXT ? WS_TEXT : ""), WS_TEXTMAXLEN, oaction, ohidden );
				htmlpage += buf;

				break;
/* \\\ WSTYPE_TEXT / PASSWORD */
/* /// WSTYPE_NUMBER
*/
/***************************************************************************/
			case WSTYPE_NUMBER:
				debugex(( "HTML_CreateUIFromTable() - WSTYPE_NUMBER\n" ));
				if( ( serverarg != nullptr ) ) {
					value = serverarg.toInt();
					debugex(( "HTML_CreateUIFromTable() - WSTYPE_NUMBER: Arg is %ld\n", (long int) value ));
					/* make sure the new value is within the allowed limits */
					if( value > WS_NUMBERMAX ) {
						value = WS_NUMBERMAX;
					}
					if( value < WS_NUMBERMIN ) {
						value = WS_NUMBERMIN;
					}
					*WS_NUMBERPTR = value;
				}
				/* build web page here */
				HTML_CreateTD( WSF_R );
				htmlpage += oname;
				htmlpage += "</td>";
				HTML_CreateTD( WSF_L );

#define DOM_NM  "<input type=\"number\" id=\"%s\" name=\"%s\" value=%ld min=%ld max=%ld %s>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_NM, oid, oid, (long int) *WS_NUMBERPTR, (long int) WS_NUMBERMIN, (long int) WS_NUMBERMAX, oaction );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_NUMBER */
/* /// WSTYPE_COLOR
*/
/***************************************************************************/
			case WSTYPE_COLOR:
				debugex(( "HTML_CreateUIFromTable() - Color mode\n" ));
				if( ( serverarg != nullptr ) ) {
					struct CRGB *t = (struct CRGB *) WS_COLORRGB;
					if( serverarg[0] == '#' ) {
						debug(( "HTML_CreateUIFromTable() - WSTYPE_COLOR: Arg is %s\n", serverarg.c_str() ));
							t->red   = ( HTML_CharToHex( serverarg[ 1 ] ) << 4 ) | HTML_CharToHex( serverarg[ 2 ] );
							t->green = ( HTML_CharToHex( serverarg[ 3 ] ) << 4 ) | HTML_CharToHex( serverarg[ 4 ] );
							t->blue  = ( HTML_CharToHex( serverarg[ 5 ] ) << 4 ) | HTML_CharToHex( serverarg[ 6 ] );
					} else {
						debug(( "HTML_CreateUIFromTable() - Color mode: Arg is invalid\n" ));
					}
				}
				/* build web page here */

				if( !td ) {
					HTML_CreateTD( WS_FLAGS );
					td++;
				}
				{
				struct CRGB *color = (struct CRGB *) WS_COLORRGB;

#define DOM_CO  "<input type=\"color\" id=\"%s\" name=\"%s\" value=\"#%02x%02x%02x\" %s>"

					snprintf( buf, BUFFER_SIZEOF-1, DOM_CO,  oid,  oid, color->red, color->green, color->blue, oaction );
					htmlpage += buf;
				}
				break;
/* \\\ WSTYPE_COLOR */
/* /// WSTYPE_YESNO
*/
/***************************************************************************/
			case WSTYPE_YESNO:
			case WSTYPE_ONOFF:
				if( ( serverarg != nullptr ) ) {
					debug(( "HTML_CreateUIFromTable() - WSTYPE_YESNO : %s = %s\n", oid, serverarg.c_str() ));
					if( ( serverarg == "On"  ) ||
						( serverarg == "Yes" ) ){
						*WS_ONOFFPTR |=  WS_ONOFFMASK;
					} else {
						*WS_ONOFFPTR &= ~WS_ONOFFMASK;
					}
				}
				/* build web page here */
				HTML_CreateTD( WSF_R );
				htmlpage += oname;
				htmlpage += "</td>";
				HTML_CreateTD( WSF_L );

#define DOM_YN1 "<select id=\"%s\" %s>"
#define DOM_YN2 "<option value=\"%s\"%s>%s</option>"
#define DOM_YN3 "</select>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_YN1, oid, oaction );
				htmlpage += buf;
				/* add all options to our cycle gadget */
				for( idx = 0 ; ( cyclestr = (char *) ( WS_TYPE == WSTYPE_YESNO ?  TAB_YESNO_YESNO[ idx ] : TAB_ONOFF_ONOFF[ idx ] ) ) ; idx++ ) {
					int tmp = 0;
					if( ( ( idx == 1 ) && !(*WS_ONOFFPTR & WS_ONOFFMASK ) ) ||
						( ( idx == 0 ) &&  (*WS_ONOFFPTR & WS_ONOFFMASK ) ) ){
						tmp++;
						debugex(( "HTML_CreateUIFromTable() - %d mask %d\n", idx, (*WS_ONOFFPTR & WS_ONOFFMASK ) ));
					}
					snprintf( buf, BUFFER_SIZEOF-1, DOM_YN2, cyclestr, ( tmp ? " selected" : "" ), cyclestr );
					htmlpage += buf;
				}
				htmlpage += DOM_YN3;
				break;
/* \\\ WSTYPE_YESNO */
/* /// WSTYPE_REFFLAG 
*/
/***************************************************************************/
			case WSTYPE_REFFLAG:
				if( ( serverarg != nullptr ) ) {
					debug(( "HTML_CreateUIFromTable() - WSTYPE_REFFLAG: Arg is %s\n", serverarg.c_str() ));
					if( ( serverarg == "1"  ) ) {
						*WS_REFFLAGPTR |=   (1 << WS_REFFLAGREF );
					} else {
						*WS_REFFLAGPTR &= ~ (1 << WS_REFFLAGREF );
					}
				}
				htmlpage += "<td></td>";
				HTML_CreateTD( WS_FLAGS );

#define DOM_RF "<input id=\"%s\" %s type=\"checkbox\" %s><label for=\"%s\">%s</label><br>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_RF, oid, (*WS_REFFLAGPTR & (1<<WS_REFFLAGREF) ) ? "checked" : "", oaction, oid, oname );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_REFFLAG */ 
/* /// WSTYPE_LABEL
*/
/***************************************************************************/
			case WSTYPE_LABEL:
				HTML_CreateTD( ( WS_FLAGS & (WSF_C|WSF_L|WSF_R) ) ? WS_FLAGS : WSF_R );

#define DOM_LA  "<label id=\"%s\" for=\"%s\">%s</label>"

				snprintf( buf, BUFFER_SIZEOF-1, DOM_LA, oid, (WS_LABELFOR ? WS_LABELFOR : ""), oname );
				htmlpage += buf;
				break;
/* \\\ WSTYPE_LABEL */
/* /// WSTYPE_HSPACE
*/
/***************************************************************************/
			case WSTYPE_HSPACE:
				htmlpage += "<td>";
				break;
/* \\\ WSTYPE_HSPACE */
/* /// WSTYPE_DATA 
*/
/***************************************************************************/
			case WSTYPE_PLAIN:
				htmlpage += oid;
				break;
/* \\\ WSTYPE_DATA */
/* /// WSTYPE_DEFAULT
*/
/***************************************************************************/
			default:
				debug(( "HTML_CreateUIFromTable() - Unknown mode\n" ));
				break;
/* \\\ */
		}
		if( !( WS_FLAGS & ( WSF_ST | WSF_H ) ) ) {
 			htmlpage += "</td>"; /* tab done */
			td = 0;
		}
		if( !( WS_FLAGS & WSF_SR ) ) {
 			htmlpage += "</tr>"; /* row done */
			tr = 0;
		}
	}
	htmlpage += "</table>";
	return( update );
}
/* \\\ */