/*
**    rl_led.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#define DISABLE_DEBUG     1  /* this overrides the global debug */

/***************************************************************************/

#include "main.h"
#include "rl_global.h"
#include "rl_wifi.h"

/***************************************************************************/

/*
** some define for more readble code 
*/

#define LED_ON                      LOW
#define LED_OFF                     HIGH

/***************************************************************************/

/*
** disable debug locally if requested
*/

#if DISABLE_DEBUG
#undef  debug
#define debug(x)
#endif

/***************************************************************************/

/* /// LED_Setup()
**
*/

/***************************************************************************/

void LED_Setup( void )
{
	pinMode( HARDWARE_ONBOARDLED_PIN, OUTPUT );
	digitalWrite( HARDWARE_ONBOARDLED_PIN, LED_OFF );      /* turn LED off */
}
/* \\\ LED_Setup */


/* /// LED_BlinkRythm() 
**
*/

/***************************************************************************/

void LED_BlinkRythm( int32_t micros, uint32_t speedon, uint32_t speedoff, uint32_t rythm )
{
uint32_t trythm;

	debug(( "\nLED_BlinkRythm() - micros %dms, Speed: %dms -> Blinking: %d times\n", micros, speedon, speedoff ));

	while( micros > 0 ) {   /* time left, so keep blinking */
		trythm = rythm; /* restart pattern */
		debug(( "Rythm: " ));
		for( int i = 0 ; trythm ; i++ ) {
			if( trythm & ( 1 << i ) ) {
				trythm &=  ~( 1 << i );   /* clear rythm bit */
				digitalWrite( HARDWARE_ONBOARDLED_PIN, LED_ON );      /* turn LED on */
				yield();
				delay( speedon );
				micros -= speedon;
				debug(( "1" ));
			} else {
				digitalWrite( HARDWARE_ONBOARDLED_PIN, LED_OFF );      /* turn LED off */
				yield();
				delay( speedoff );
				micros -= speedoff;
				debug(( "0" ));
			}
		}
		debug(( "\n" ));
		digitalWrite( HARDWARE_ONBOARDLED_PIN, LED_OFF );      /* turn LED off */
	}
}
/* \\\ LED_BlinkRythm */

/***************************************************************************/
