/*
**   rl_sound.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_global.h"
#include "rl_sound.h"
#include "rl_time.h"

#include <DFRobotDFPlayerMini.h>

/***************************************************************************/

HardwareSerial SoundSerial(1); /* RX, TX */
DFRobotDFPlayerMini SoundPlayer;

uint32_t volumecurrent;         /* current volume in hardware, we could read it, but some hardware may not support this */

/***************************************************************************/

/* /// Sound_Setup()
**
*/

/***************************************************************************/

void Sound_Setup( void )
{
	debug(( "\nSound_Startup()\n" ));

	volumecurrent = 999; /* set an invalid number to force hardware update */

	SoundSerial.begin( SOUND_SERIALSPEED, SOUND_SERIALFLAGS, SOUND_RX_PIN , SOUND_TX_PIN  );

	SoundPlayer.begin( SoundSerial );
	SoundPlayer.setTimeOut( SOUND_INITDELAY );
	SoundPlayer.disableLoopAll();
	SoundPlayer.EQ( DFPLAYER_EQ_NORMAL );
	SoundPlayer.outputDevice( DFPLAYER_DEVICE_SD );

/* set volume depending on user settings and current time */

	Global_UpdateVolume();

}
/* \\\ */

/* /// Sound_Play()
**
*/

/***************************************************************************/

void Sound_Play( uint32_t soundid, bool force )
{

	debug(( "\nSound_Play() - %d\n", soundid ));

	if( ( ( rlc.rlc_Flags & RLCF_SOUNDENABLE ) && !( rlrd.rlrd_Flags & RLRDF_STATUS_GLOBALOFF ) ) || force ) {
		SoundPlayer.play( soundid );
	}

}
/* \\\ Sound_Play */
/* /// Sound_SetVolume()
**
** This function set the time. It takes care of the current volume and
** the new volume, so it does not matter if it got called ten times
** in a row. 
**
** The player hardware is only accessed when the new volume is different.
**
*/

/***************************************************************************/

void Sound_SetVolume( uint32_t volume )
{

/* make sure we are within limits */

	if( volume > 100 ) {
		volume = 100;
	}

/* only access hardware when really needed */

	if( volumecurrent != volume ) {
		volumecurrent = volume;   /* store data so we can avoid setting the same volume twice */

		debug(( "Sound_SetVolume() - Setting %d%% (%d)\n", volume, volume * SOUNDDEVICE_VOLUME_MAX / 100 ));

		volume = volume * SOUNDDEVICE_VOLUME_MAX / 100; /* convert % into real world data */

		SoundPlayer.volume( volume );
	}
}
/* \\\ Sound_SetVolume */